from nomad_media_cli.helpers.utils import initialize_sdk

import click
import os
import sys
import threading

@click.command()
@click.option("--file", help="File to upload")
@click.option("--id", help="Parent folder id")
@click.option("--path", help="Is the objectKey (which defaults to the default content bucket) or the url to the folder.")
@click.option("-r", "--recursive", is_flag=True, help="Recursively upload a folder")
@click.pass_context
def upload_assets(ctx, file, id, path, recursive):
    """Upload assets"""
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        parent_id = None        

        if not file:
            click.echo({ "error": "Please provide a file to upload." })
            sys.exit(1)
            
        if not id and not path:
            click.echo({ "error": "Please provide a parent id or path." })
            sys.exit(1)
            
        if path and "::" not in path:
            if not "bucket" in ctx.obj:
                click.echo("Please provide a valid path or set the default bucket.")
                sys.exit(1)
            path = f"{ctx.obj['bucket']}::{path}"
            
        if path:
            parent_id = path
        elif id:
            parent_id = id
            
        if os.path.isdir(file):
            if recursive:
                threads = []
                for root, dirs, files in os.walk(file):
                    folder_name = os.path.basename(root)
                    
                    folder = None
                    while True:
                        nomad_folders = nomad_sdk.search(None, None, None,
                            [
                                {
                                    "fieldName": "assetId",
                                    "operator": "equals",
                                    "values": parent_id
                                },
                                {
                                    "fieldName": "assetType",
                                    "operator": "equals",
                                    "values": 1
                                }
                            ],
                            None, None, None, None, None, None, None, None, None, None, None)
                        
                        if len(nomad_folders["items"]) == 0:
                            break
                        
                        folder = next((nomad_folder for nomad_folder in nomad_folders["items"] if nomad_folder["title"] == folder_name), None)
                        if folder:
                            break
                        
                    if not folder:
                        folder = nomad_sdk.create_folder_asset(parent_id, folder_name)
                        
                    folder_id = folder["id"]
                    parent_id = folder_id

                    for name in files:
                        file_path = os.path.join(root, name)
                        thread = threading.Thread(target=nomad_sdk.upload_asset, args=(None, None, None, "replace", file_path, folder_id, None))
                        thread.start()
                        threads.append(thread)
                        
                for thread in threads:
                    thread.join()
                    
            else:
                click.echo({ "error": "Please use the --recursive option to upload directories." })
                sys.exit(1)
        else:
            nomad_sdk.upload_asset(None, None, None, "replace", file, parent_id, None)
                
    except Exception as e:
        click.echo({ "error": f"Error uploading assets: {e}" })
        sys.exit(1)


import click
import json
import os

@click.command()
@click.option("--service-api-url", required=True, help="API URL for the service")
@click.option("--api-type", default="admin", type=click.Choice(['admin', 'portal']), help="API type (i.e. admin, portal )")
@click.option("--debug-mode", default="false", type=click.Choice(['true', 'false']), help="Enable debug mode")
@click.option("--singleton", default="true", type=click.Choice(['true', 'false']), help="Enable singleton mode")
@click.pass_context
def init(ctx, service_api_url, api_type, debug_mode, singleton):
    """Initialize the SDK and save configuration"""
    
    config_path = ctx.obj["config_path"]
    config_dir = os.path.dirname(config_path)
    os.makedirs(config_dir, exist_ok=True)
    
    config = {
        "serviceApiUrl": service_api_url,
        "apiType": api_type,
        "debugMode": False,
        "singleton": singleton == "true",
        "disableLogging": True
    }
    
    try:
        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
    
    except Exception as e:
        click.echo({ "error": f"Error saving configuration: {e}" })


from nomad_media_cli.helpers.utils import initialize_sdk

import click
import json
import sys

@click.command()
@click.option("--id", help="Can be an assetId (file), an assetId (folder), a collectionId, a savedSearchId (lower priority).")
@click.option("--path", help="Is the objectKey (which defaults to the default content bucket) or the url to the folder.")
@click.pass_context
def list_assets(ctx, id, path):
    """List assets"""
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:        
        filter = None
        if id: 
            filter = [{
                "fieldName": "uuidSearchField",
                "operator": "equals",
                "values": id
            }]
        elif path:
            if "::" not in path:
                if not "bucket" in ctx.obj:
                    click.echo("Please provide a valid path or set the default bucket.")
                    sys.exit(1)
                path = f"{ctx.obj['bucket']}::{path}"

            filter = [{
                "fieldName": "url",
                "operator": "equals",
                "values": path
            }]
        else:
            click.echo("Please provide an id or path.")
            sys.exit(1)

        results = nomad_sdk.search(None, None, 5, filter, 
            [
                { 
                    "fieldName": "identifiers.url", 
                    "sortType": "ascending"
                }
            ], 
            [
                { "name": "id"},
                { "name": "identifiers.name"},
                { "name": "identifiers.url"},
                { "name": "identifiers.fullUrl"},
                { "name": "identifiers.assetTypeDisplay"},
                { "name": "identifiers.mediaTypeDisplay"},
                { "name": "identifiers.contentLength"}
            ], None, None, None, None, None, None, None, None, None)
        
        click.echo(json.dumps(results["items"], indent=4))
    
    except Exception as e:
        click.echo({ "error": f"Error listing assets: {e}" })
        sys.exit(1)


import click
import json
import sys

@click.command()
@click.option("--service-api-url", help="API URL for the service")
@click.option("--api-type", default="admin", type=click.Choice(['admin', 'portal']), help="API type (i.e. admin, portal )")
@click.option("--debug-mode", default="false", type=click.Choice(['true', 'false']), help="Enable debug mode")
@click.option("--singleton", default="true", type=click.Choice(['true', 'false']), help="Enable singleton mode")
@click.pass_context
def update_config(ctx, service_api_url, api_type, debug_mode, singleton):
    """Update the configuration"""
    
    config_path = ctx.obj["config_path"]

    try:
        with open(config_path, "r") as file:
            config = json.load(file)
        
        if service_api_url:
            config["serviceApiUrl"] = service_api_url
        if api_type:
            config["apiType"] = api_type
        if debug_mode:
            config["debugMode"] = debug_mode == "true"
        if singleton:
            config["singleton"] = singleton == "true"

        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
    
    except Exception as e:
        click.echo({ "error": f"Error updating configuration: {e}" })
        sys.exit(1)


import click
import json
import sys

@click.command()
@click.option("--tag", required=True, multiple=True, help="tag name")
@click.pass_context
def set_tags(ctx , tag):
    """Set the default tag"""

    config_path = ctx.obj.get("config_path")

    try:
        with open(config_path, "r") as file:
            config = json.load(file)
        
        config["tag"] = ",".join(tag)

        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
    
    except Exception as e:
        click.echo({ "error": f"Error setting tag: {e}" })       
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk

import click
import json
import sys

@click.command()
@click.option("--size", default=10, help="The number of results shown. Default is 10.")
@click.option("--offset", help="The offset of the page.")
@click.pass_context
def list_tags(ctx, size, offset):
    """List tags"""    
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:    
        filter = [{
            "fieldName": "contentDefinitionId",
            "operation": "equals",
            "values": "c806783c-f127-48ae-90c9-32175f4e1fff"
        }]

        results = nomad_sdk.search(None, offset, size, filter, None, None, None, None, None, None, None, None, None, None, None)

        result_names = [result["title"] for result in results["items"]]
        
        click.echo(json.dumps(result_names, indent=4))

    except Exception as e:
        click.echo({ "error": f"Error listing tags: {e}" })
        sys.exit(1)


import click

import os
from platformdirs import user_config_dir

from nomad_media_cli.commands.init import init
from nomad_media_cli.commands.list_assets import list_assets
from nomad_media_cli.commands.list_buckets import list_buckets
from nomad_media_cli.commands.list_config_path import list_config_path
from nomad_media_cli.commands.list_tags import list_tags
from nomad_media_cli.commands.login import login
from nomad_media_cli.commands.set_bucket import set_bucket
from nomad_media_cli.commands.set_tags import set_tags
from nomad_media_cli.commands.update_config import update_config
from nomad_media_cli.commands.upload_assets import upload_assets

from nomad_media_cli.helpers.check_token import check_token

# Set the configuration directory and path
CONFIG_DIR = user_config_dir("nomad_media_cli")
CONFIG_PATH = os.path.join(CONFIG_DIR, "config.json")

@click.group()
@click.option("--config-path", default=CONFIG_PATH, help="Path to the configuration file (optional)")
@click.pass_context
def cli(ctx, config_path):
    """Nomad Media CLI"""
    ctx.ensure_object(dict)
    ctx.obj["config_path"] = config_path    
    
cli.add_command(init)
cli.add_command(list_assets)
cli.add_command(list_buckets)
cli.add_command(list_config_path)
cli.add_command(list_tags)
cli.add_command(login)
cli.add_command(set_bucket)
cli.add_command(set_tags)
cli.add_command(update_config)
cli.add_command(upload_assets)

@cli.result_callback()
@click.pass_context
def process(ctx, *args, **kwargs):
    if ctx.obj.get("nomad_sdk"):
        check_token(ctx.obj["config_path"], ctx.obj["nomad_sdk"])

if __name__ == "__main__":
    cli(obj={})
